<?php

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;


class YooMoney
{
    public const TYPE_OPERATIONS = ['deposition', 'payment'];

    protected Client $client;

    public function __construct(string $token)
    {
        $this->client = new Client([
          'base_uri'        => 'https://yoomoney.ru/api/',
          'headers'         => [
            'Content-Type'    => 'application/x-www-form-urlencoded',
            'Authorization'   => 'Bearer ' . $token,
            'Accept'          => 'application/json',
            'Accept-Charset'  => 'utf-8',
            'Accept-Language' => 'ru',
            'Accept-Encoding' => 'gzip',
            'Cache-Control'   => 'no-cache',
            'Pragma'          => 'no-cache',
            'User-Agent'      => NULL,
          ],
          'verify'          => true,
          'allow_redirects' => false,
          'http_errors'     => false,
          'timeout'         => 10,
          'connect_timeout' => 3,
          'read_timeout'    => 5,
          'synchronous'     => true,
          'delay'           => 250,
        ]);
    }

    public function getInfo(): array
    {
        return $this->request('account-info');
    }

    protected function request($path, $params = []): array
    {
        $options = empty($params) ? [] : ['form_params' => $params];

        $response = $this->client->post($path, $options);

        if($response->getStatusCode() >= 299) {
            throw new Exception('Response code is ' . $response->getStatusCode(), $response->getStatusCode());
        }

        $response = json_decode($response->getBody()->getContents(), true);

        if(!empty($response['error'])) {
            throw new Exception($response['error']);
        }

        return $response;
    }

    public function getOperation(string $id): array
    {
        $params = [
          'operation_id' => $id,
        ];

        $response = $this->request('operation-details', $params);

        $response['datetime'] = new DateTime($response['datetime']);

        return $response;
    }

    public function getOperations(DateTime $from = NULL, DateTime $to = NULL, string $type = NULL, string $label = NULL, int $offset = 0): array
    {
        $params = [
          'records' => 100,
          'details' => 'true',
        ];

        if(!is_null($type)) {
            if(in_array($type, self::TYPE_OPERATIONS)) {
                $params['type'] = $type;
            } else {
                throw new Exception('Type must be `deposition` or `payment`');
            }
        }

        if(!is_null($label)) {
            $params['label'] = $label;
        }

        if($offset > 0) {
            $params['start_record'] = $offset;
        }

        if(!is_null($from)) {
            $params['from'] = $from->format(DateTime::RFC3339);
        }

        if(!is_null($to)) {
            $params['till'] = $to->format(DateTime::RFC3339);
        }

        $response = $this->request('operation-history', $params);

        $response['operations'] = array_map(function($item) {
            $item['datetime'] = new DateTime($item['datetime']);

            return $item;
        }, $response['operations']);

        return $response;
    }

    public function confirm(string $requestId): array
    {
        $params = [
          'request_id'   => $requestId,
          'money_source' => 'wallet',
        ];

        return $this->request('process-payment', $params);
    }

    public function transfer(string $to, float $amount, string $message = NULL, string $comment = NULL, string $label = NULL): array
    {
        $params = [
          'pattern_id' => 'p2p',
          'to'         => $to,
          'amount'     => $amount,
        ];

        if(!is_null($message)) {
            $params['message'] = $message;
        }

        if(!is_null($comment)) {
            $params['comment'] = $comment;
        }

        if(!is_null($label)) {
            $params['label'] = $label;
        }

        return $this->request('request-payment', $params);
    }

}